/*
 * @version 0.0 17.12.2008
 * @author 	Tobse F
 */
package satteliteGame;

import lib.mylib.SpriteSheet;
import lib.mylib.object.BasicGameState;
import lib.mylib.object.SObject;
import lib.mylib.object.SObjectList;
import lib.mylib.util.MusicJukebox;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.Music;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;


public class MainGame extends BasicGameState{

	private Earth earth;
	private Sattelite sattelite;
	private WolkFactory wolkFactory;
	private SObjectList gameObjects;
	private BarSpeed speed;
	private BarPower energy;
	private ShootingStarFactory stars;
	private StarFactory starFactory;
	private Thermometer thermometer;
	private Pause pause;
	private Font arial_black_71, digitdb_23, arial_bold_26;
	private SurviveClock clock;
	private MusicJukebox jukebox;
	private CollisionMechanic collisionMechanic;
	private WarminMechanic warminMechanic;
	private Wolkometer wolkometer;
	private GameOver gameOver;
	private GameContainer container; 
	private StateBasedGame game;
 
	public MainGame(int id) {
		super(id);
	}
	
	@Override
	public void init(GameContainer container, StateBasedGame game) throws SlickException {
		this.container = container;
		this.game = game;
		String res = GameProprties.RES_DIR;
		arial_black_71 = new AngelCodeFont(res + "arial_black_71.fnt",
				res+ "arial_black_71.png");
		digitdb_23 = new AngelCodeFont(res + "digitdb_32.fnt",
				res+ "digitdb_32.png");
		arial_bold_26 = new AngelCodeFont(res + "arial_bold_26.fnt",
				res+ "arial_bold_26.png");
		gameObjects = new SObjectList();
		earth = new Earth(new Image(res + "earth.png"), GameProprties.EARTH_POS_X, GameProprties.EARTH_POS_Y);
		sattelite = new Sattelite(earth, container.getInput());
		earth.setEarthShadow( new Image(res+"earth_shadow.png"));
		sattelite.setImage(new Image(res + "sattelite.png"));
		wolkFactory = new WolkFactory();
		clock = new SurviveClock(digitdb_23, 900, 715);
		speed = new BarSpeed(800, 25, 180, 40, Color.red);
		energy = new BarPower(800, 90, 180, 40, Color.yellow);
		energy.setValue(1f);
		speed.setRange(0f, (float)GameProprties.SATTELITE_MAX_SPEED);
		speed.setValue(0.3f);
		sattelite.addEventListener(speed);
		sattelite.setBar(energy);
		
		thermometer = new Thermometer(8, 360, new Image(res + "thermometer.png"));
		wolkometer = new Wolkometer(12, 16,new SpriteSheet( 
				new Image(res + "wolkometer.png"), 137, 63));
		pause = new Pause(arial_black_71, container.getWidth(), container.getHeight());
		stars = new ShootingStarFactory();
		starFactory = new StarFactory();
		
		collisionMechanic= new CollisionMechanic(sattelite, wolkFactory);
		warminMechanic = new WarminMechanic(wolkFactory, earth, wolkometer, thermometer);
		
		gameOver = new GameOver(new Image(res+"gameover.png"), digitdb_23, arial_bold_26, 
				container.getWidth(), container.getHeight());
		
		gameObjects.add(new SObject(new Image(res+"background2.jpg")));
		gameObjects.add(stars);
		gameObjects.add(starFactory);
		gameObjects.add(earth);
		gameObjects.add(sattelite);
		gameObjects.add(wolkFactory);
		gameObjects.add(speed);
		gameObjects.add(energy);
		gameObjects.add(wolkometer);
		gameObjects.add(thermometer);
		gameObjects.add(clock);
		gameObjects.add(warminMechanic);
		gameObjects.add(pause);
		gameObjects.add(gameOver);
		
		jukebox = new MusicJukebox(new Music(res + "2.mod"), new Music(res + "1.mod"));
		jukebox.setVolume(0.4f);
		jukebox.reset();
		container.setPaused(false);
	}
	
	@Override
	public void update(GameContainer container, StateBasedGame game, int delta)
			throws SlickException {
		Input in = container.getInput();
		
		if(in.isKeyPressed((Input.KEY_P))){
			container.setPaused(!container.isPaused());
		}
		if(in.isKeyPressed((Input.KEY_ENTER))){
			if(gameOver.isVisible()){
				newGame();
			}
		}
		if(!gameOver.isVisible()){
			pause.setVisible(container.isPaused());
		}
		if(!container.isPaused()){
			if(in.isKeyPressed(Input.KEY_ESCAPE)){
				container.exit();
			}
			if(in.isKeyPressed((Input.KEY_F))){
				container.setShowFPS(!container.isShowingFPS());
			}
			if(in.isKeyDown(Input.KEY_SPACE)){
				if(sattelite.hasEnougEnergy())
					collisionMechanic.update(delta);
			}			
			if(in.isKeyPressed((Input.KEY_N))){
				newGame();
			}
			
			if(sattelite.isInSun())
				energy.setValue(energy.getValue() + GameProprties.ENERGY_REGENERATION);
			
			if(!container.isPaused())
				gameObjects.update(delta);
		}
		if(warminMechanic.isGameOver())
			gameOver(container);
		
		
	}
	
	@SuppressWarnings("unused")
	private void cheats(Input in){
		if(in.isKeyPressed(Input.KEY_ENTER)){
			wolkFactory.addNewWolk();
		}
		if(in.isKeyDown((Input.KEY_A))){
			earth.warmUp();
		}
		if(in.isKeyPressed((Input.KEY_C))){
			energy.setValue(1);
		}
		
		if(in.isKeyPressed((Input.KEY_V))){
			wolkometer.setValue(wolkometer.getValue() + 0.2);
		}
		if(in.isKeyPressed(Input.KEY_ENTER)){
			wolkFactory.addNewWolk();
		}
		if(in.isKeyPressed(Input.KEY_B)){
			thermometer.setValue(thermometer.getValue() + 0.01f);
		}
		if(in.isKeyPressed((Input.KEY_G))){
			gameOver(container);
		}
	}
	
	private void gameOver(GameContainer container){
		gameOver.setVisible(true);
		container.setPaused(true);
		gameOver.setScore(wolkFactory.getGeneratedWolks(), 
				wolkFactory.getRemovedWolks(), clock.getFormattedTimeAsString());
	}

	@Override
	public void render(GameContainer container, StateBasedGame game, Graphics g)
			throws SlickException {
			gameObjects.render(g);
	}
	
	private void newGame(){
		gameObjects.reset();
		try {
			init(container, game);
		} catch (SlickException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void enter(GameContainer container, StateBasedGame game) throws SlickException {
		super.enter(container, game);
		jukebox.play();
	}
}