/*
 * @version 0.0 17.12.2008
 * @author 	Tobse F
 */
package satteliteGame;

import java.awt.Point;
import java.io.IOException;

import javax.swing.event.EventListenerList;

import lib.mylib.hacks.SimpleValue;
import lib.mylib.math.Range;
import lib.mylib.math.RangeSlider;
import lib.mylib.object.SObject;
import lib.mylib.object.ValueChangedListener;
import lib.mylib.object.ValueEvent;

import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.particles.ConfigurableEmitter;
import org.newdawn.slick.particles.ParticleIO;
import org.newdawn.slick.particles.ParticleSystem;

public class Sattelite extends SObject implements Collisionable{
	private Earth earth;
	private Bar energy;
	private EventListenerList eventListenerList = new EventListenerList();
	enum MovingType{LEFT, RIGHT, UP, DOWN};
	private MovingType movingDirection;
	private ParticleSystem stream, brush, warning_light;
	private static final float ROTATION_SPEED = 0.5f;
	private Color lighning;
	private ConfigurableEmitter[] spots = new ConfigurableEmitter[2];
	private static final int COLLISION_CIRCLE_RADIUS = 20;
	private RangeSlider height;
	private Input input;
	private RangeSlider speed;
	private double degree = 0; //TODO: change to RangeSlider -RangeSlieder need infinite!
	private Range sunside = new Range(170, 290);
	
	public Sattelite(Earth earth, Input input){
		this.earth = earth;
		this.input = input;
		speed = new RangeSlider(GameProprties.SATTELITE_MIN_SPEED, 
				GameProprties.SATTELITE_MAX_SPEED, 0.02);
		movingDirection = MovingType.LEFT;
		ParticleSystem.setRelativePath("res");
		try {
			stream = ParticleIO.loadConfiguredSystem(GameProprties.RES_DIR + "StreamSytsem.xml");
			brush = ParticleIO.loadConfiguredSystem(GameProprties.RES_DIR + "BrushSystem.xml");
			warning_light = ParticleIO.loadConfiguredSystem(GameProprties.RES_DIR + "red_light.xml");
		} catch (IOException e) {}
		lighning = new Color(Color.yellow);
		height = new RangeSlider(60, 195, GameProprties.SATTELITE_HIGH_SPEED, 80);
		spots[0] = ((ConfigurableEmitter)stream.getEmitter(0));		
		spots[1] = spots[0].duplicate();
		stream.addEmitter(spots[1]);
	}
	
	public void addEventListener(ValueChangedListener valueListener){
		eventListenerList.add(ValueChangedListener.class, valueListener);
	}
	
	@Override
	public void render(Graphics g) {
		if(isInSun())
			image.draw(x - image.getWidth() / 2, y - image.getHeight() / 2, lighning);
		else
			image.drawCentered(x, y);
		stream.render();
		brush.render();
		warning_light.render();
		//g.fill(getCollisionCircles()[0]);
	}
	
	private Point calculatePosition(){
		int x,y;
		x =(int)((height.getValue() + Earth.EARTH_R) * Math.sin(Math.toRadians(degree)));
		y = (int)((height.getValue() + Earth.EARTH_R) * Math.cos(Math.toRadians(degree)));
		return new Point(x,y);
	}

	@Override
	public void update(int delta) {
		setStreaming(input.isKeyDown((Input.KEY_LEFT)) ||
				input.isKeyDown(Input.KEY_RIGHT), 0);
		setStreaming(input.isKeyDown((Input.KEY_UP)) ||
				input.isKeyDown(Input.KEY_DOWN ),1);
		
		if(input.isKeyDown((Input.KEY_UP))){
			push(MovingType.UP);
		}
		if(input.isKeyDown((Input.KEY_DOWN))){
			push(MovingType.DOWN);
		}	
		if(input.isKeyDown((Input.KEY_LEFT))){
			push(MovingType.LEFT);
		}
		if(input.isKeyDown(Input.KEY_RIGHT)){
			push(MovingType.RIGHT);
		}
		setBrushing(input.isKeyDown(Input.KEY_SPACE));
		stream.update(delta);
		brush.update(delta);
		warning_light.update(delta);
		image.rotate(ROTATION_SPEED);
		Point newPosition = calculatePosition();		
		newPosition.translate(earth.getX(),earth.getY());
		
		x = newPosition.x;
		y = newPosition.y;
		stream.setPosition(x, y);
		((ConfigurableEmitter)brush.getEmitter(0)).setPosition(x, y);
		warning_light.setPosition(x, y);

		if(movingDirection == MovingType.LEFT){
			degree = (degree + speed.getValue() > 360) ? 0 : degree + speed.getValue();
		}else{
			degree = (degree - speed.getValue() < 0) ? 360 : degree - speed.getValue();
		}
		if(!hasEnougEnergy()){
			setStreaming(false , 0);
			setStreaming(false , 1);
		}
		((ConfigurableEmitter)warning_light.getEmitter(0)).setEnabled(!hasEnougEnergy());
	}
	
	private void speedUp(){
		speed.stepRight();
	}
	private void speedDown(){
		speed.stepLeft();
	}

	public double getDegree() {
		return degree;
	}
	
	public void setBar(Bar bar) {
		this.energy = bar;
	}
	
	public void push(MovingType movingType){		
		if(energy.getValue() > speed.getStep()){;
			int rotDegree = 180 - (int)this.degree;
			if(movingType == movingDirection){
				speedUp();
			}else if (movingType == MovingType.LEFT || movingType == MovingType.RIGHT){
				speedDown();
				if(speed.getValue() <= GameProprties.SATTELITE_MIN_SPEED){
					toggleMovingDirection();
				}
			}
			switch (movingType) {
			case RIGHT:
				spots[0].angularOffset = new SimpleValue(rotDegree -90) ;
				break;
			case LEFT:
				spots[0].angularOffset = new SimpleValue(rotDegree +90) ;
				break;
			case DOWN:
				spots[1].angularOffset = new SimpleValue(rotDegree);
				height.stepLeft();
				break;
			case UP:
				spots[1].angularOffset = new SimpleValue(rotDegree +180) ;
				height.stepRight();
				break;
			default:
				break;
			}
			notifyAdvertisement(new ValueEvent(this, speed.getValue()));
		}
	}
	
	public boolean isInSun(){
		return sunside.isInRange(degree);
	}
	
	protected synchronized void notifyAdvertisement( ValueEvent event ){ 
		for ( ValueChangedListener l : eventListenerList.getListeners( ValueChangedListener.class ) ){
			l.valueEvent(event);
		}
	} 
	
	private void setStreaming(boolean streaming, int engineNr) {
		if(!hasEnougEnergy()) streaming = false;
		spots[engineNr].setEnabled(streaming);
		if(streaming)
			energy.setValue(energy.getValue() - GameProprties.ENERGY_PER_PUSH);
	}
	
	public boolean hasEnougEnergy(){
		return energy.getValue() > GameProprties.ENERGY_PER_BRUSH;
	}
	
	public void setBrushing(boolean brushing) {
		if(!hasEnougEnergy()) brushing = false;
		((ConfigurableEmitter)brush.getEmitter(0)).setEnabled(brushing);
		if(brushing)
			energy.setValue(energy.getValue() - GameProprties.ENERGY_PER_BRUSH);
	}
	
	private void toggleMovingDirection(){
		movingDirection = (movingDirection == MovingType.RIGHT) ?
				MovingType.LEFT : MovingType.RIGHT;
	}

	@Override
	public Circle[] getCollisionCircles() {
		return new Circle[]{new Circle(x, y, COLLISION_CIRCLE_RADIUS)};
	}
	
	@Override
	public void collide() {
		setVisible(false);
	}
	
}