/*
 * @version 0.0 19.12.2008
 * @author 	Tobse F
 */
package satteliteGame;

import java.util.Random;

import lib.mylib.ValueSlider;
import lib.mylib.ValueSlider.ValueSliderType;
import lib.mylib.math.MyMath;
import lib.mylib.object.SObject;

import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Transform;

public class Wolk extends SObject implements Collisionable{

	private Color color;
	private int skyRadius;
	private double degree;
	/** Speed in Degrees	 */
	private float movingSpeed;
	private Shape position;
	private int collisionRadius = 24;
	private boolean isDisbanding = false;
	private ValueSlider tranparency;
	
	
	public Wolk(Image image, int height, double degree) {
		movingSpeed = MyMath.getFloat(0f, 0.01f);
		position = new Circle(0,0, collisionRadius);
		this.image = image;
		skyRadius = Earth.EARTH_R  + height;
		this.degree = degree;
		position.setLocation(GameProprties.EARTH_POS_X + skyRadius, GameProprties.EARTH_POS_Y);
		position = position.transform(Transform.createRotateTransform(
				(float)Math.toRadians(degree), 
				GameProprties.EARTH_POS_X, GameProprties.EARTH_POS_Y));
		if( new Random().nextBoolean()){
			color = Color.gray.brighter(MyMath.getFloat(0f, 0.6f));
		}else{
			color = Color.gray.darker(MyMath.getFloat(0f, 0.6f));
		}
		color.a = 0.9f;
		tranparency = new ValueSlider(0,0.9,50,MyMath.getInt(1500, 3000),
				ValueSliderType.SLIDE_IN);
		update(0);
	}
	
	@Override
	public void render(Graphics g) {
		g.drawImage(image, x, y, color);
	}
	
	public double getDegree() {
		return degree;
	}
	
	@Override
	public void update(int delta) {
		degree += movingSpeed;
		position = position.transform(Transform.createRotateTransform(
						(float) Math.toRadians(movingSpeed), GameProprties.EARTH_POS_X,
						GameProprties.EARTH_POS_Y));
		x = (int) position.getCenterX();
		y = (int) position.getCenterY();
		image.setCenterOfRotation((float) Math.sqrt(Earth.EARTH_R),
				(float) Math.sqrt(Earth.EARTH_R));
		image.setRotation((float) degree);
		image.setCenterOfRotation(0, 0);
		image.rotate(90f);
		tranparency.update(delta);
		color.a = (float) tranparency.getValue();
		if (isDisbanding && tranparency.isFinshed()) {
			finish();
		}
	}

	@Override
	public void collide() {
		if(!isDisbanding){
			isDisbanding = true;
			tranparency.invert();
		}
	}

	@Override
	public Circle[] getCollisionCircles() {
		return new Circle[]{new Circle(
				position.getCenterX(), position.getCenterY(), collisionRadius)};
	}
}