/*
 * @version 0.0 21.12.2008
 * @author 	Tobse F
 */
package satteliteGame;

import java.util.List;

import lib.mylib.MyTimer;
import lib.mylib.SpriteSheet;
import lib.mylib.ValueSlider;
import lib.mylib.ValueSlider.ValueSliderType;
import lib.mylib.math.MyMath;
import lib.mylib.object.RemoveObjectListener;
import lib.mylib.object.SObject;
import lib.mylib.object.SObjectList;
import lib.mylib.object.SObjectRemovedEvent;

import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;

public class WolkFactory extends SObject implements RemoveObjectListener{
	private SObjectList wolks;
	private MyTimer timer;
	private SpriteSheet wolkImages;
	private ValueSlider minSpeed, maxSpeed;
	private int generatedWolks = 0, removedWolks = 0;
	
	public WolkFactory() {
		try {
			wolkImages = new SpriteSheet(
					new Image(GameProprties.RES_DIR + "wolks.png"), 79, 35);
		} catch (SlickException e) {
			e.printStackTrace();
		}
		double rangeGap = GameProprties.WOLKFACTORY_RANGE_GAP;
		minSpeed = new ValueSlider(GameProprties.WOLKFACTORY_MIN_WOLK_GENERATION_TIME, 
				GameProprties.WOLKFACTORY_MAX_WOLK_GENERATION_TIME - rangeGap, 100, 
				1000 * GameProprties.WOLKFACTORY_TIME_BEFORE_MAX_GENERATION_TIME, 
				ValueSliderType.SLIDE_OUT);
		
		maxSpeed = new ValueSlider(
				GameProprties.WOLKFACTORY_MIN_WOLK_GENERATION_TIME + rangeGap, 
				GameProprties.WOLKFACTORY_MAX_WOLK_GENERATION_TIME, 100, 
				1000 * GameProprties.WOLKFACTORY_TIME_BEFORE_MAX_GENERATION_TIME, 
				ValueSliderType.SLIDE_OUT);
		
		wolks = new SObjectList();
		wolks.addRemoveObjectListener(this);
		timer = new MyTimer(1500, true){
			@Override
			protected void timerAction() {
				addNewWolk();
			}
		};	
	}
	
	
	public void addNewWolk(){
		Wolk newWolk = new Wolk(
				wolkImages.getSprite(MyMath.getInt(0, wolkImages.getSpriteCount())),
				MyMath.getInt(55, 180), MyMath.getInt(0, 360));
		timer.setDuration(
				MyMath.getInt((int)minSpeed.getValue(),(int)maxSpeed.getValue()));
		wolks.add(newWolk);
		generatedWolks ++;
	}
	
	@Override
	public void render(Graphics g) {
		wolks.render(g);
	}
	
	@Override
	public void update(int delta) {
		wolks.update(delta);
		timer.update(delta);
		minSpeed.update(delta);
		maxSpeed.update(delta);
	}
	
	public List<SObject> getWolks() {
		return wolks.getObjectList();
	}

	public int getWolkCount(){
		return wolks.size();
	}

	@Override
	public void removedObecect(SObjectRemovedEvent objectRemovedEvent) {
		removedWolks++;
	}
	
	public int getRemovedWolks() {
		return removedWolks;
	}
	
	public int getGeneratedWolks() {
		return generatedWolks;
	}
	
}